from importlib import import_module
from contextlib import contextmanager


class InvalidEnvironmentError(Exception):
    pass


class SettingsNotActiveError(Exception):
    pass


class Settings:
    def __init__(self, module_path):
        self._module_path = module_path
        self._environment_module = None
        self.current_environment = None
        self._overrides = {}

    @property
    def _environment_path(self):
        return '.'.join([self._module_path, self.current_environment])

    def __getattr__(self, item):
        if self._environment_module is None:
            raise SettingsNotActiveError('No settings environment is active.')

        try:
            return self._overrides[item]
        except KeyError:
            return getattr(self._environment_module, item)

    @contextmanager
    def environment(self, env):
        try:
            self.current_environment = env
            try:
                self._environment_module = import_module(self._environment_path)
            except ImportError:
                raise InvalidEnvironmentError("'%s' is not a valid environment." % env)
            yield
        finally:
            self._environment_module = None
            self.current_environment = None

    @contextmanager
    def override(self, **kwargs):
        try:
            if self.current_environment is None:
                raise SettingsNotActiveError('No settings environment is active.')

            self._overrides = dict(**kwargs)

            yield
        finally:
            self._overrides = {}
